@@ -258,7 +258,7 @@ if(dot && (dbuf[0] == '0'))
  */
 static char *convert_ascii_2(Trptr t, vptr v)
 {
-Ulong flags;
+TraceFlagsType flags;
 int nbits;
 unsigned char *bits;
 char *os, *pnt, *newbuff;
@@ -696,7 +696,16 @@ else if(flags&TR_SIGNED)
 			}
 			else
 			{
-			sprintf(os, TTFormat, val);
+			if(!(flags&TR_TIME))
+				{
+				sprintf(os, TTFormat, val);
+				}
+				else
+				{
+				free_2(os);
+				os = calloc_2(1, 128);
+				reformat_time(os, val, GLOBALS->time_dimension);
+				}
 			}
 		}
 		else
@@ -793,7 +802,16 @@ else	/* decimal when all else fails */
 			}
 			else
 			{
-			sprintf(os, UTTFormat, val);
+                        if(!(flags&TR_TIME))
+                                {
+				sprintf(os, UTTFormat, val);
+                                }
+                                else
+                                {
+                                free_2(os);
+                                os = calloc_2(1, 128);
+                                reformat_time(os, val, GLOBALS->time_dimension);
+                                }
 			}
 		}
 		else
@@ -878,8 +896,8 @@ return(rv);
 
 int vtype(Trptr t, char *vec)
 {
-int i, nbits, res;
-int an_u_encountered = 0;
+int i, nbits;
+char pch, ch;
 
 if (vec == NULL)
 	return(AN_X);
@@ -887,34 +905,32 @@ if (vec == NULL)
 nbits=t->n.nd->msi-t->n.nd->lsi;
 if(nbits<0)nbits=-nbits;
 nbits++;
-res = AN_1;
-for (i = 0; i < nbits; i++)
+pch = ch = vec[0];
+for (i = 1; i < nbits; i++)
 	{
-	switch (*vec)
-		{
-		case AN_X:
-		case 'x':
-		case 'X':
-				return(AN_X);
-		case AN_U:
-		case 'u':
-		case 'U':
-				an_u_encountered = 1; break;
-		case AN_Z:
-		case 'z':
-		case 'Z':
-				if (res == AN_0) return(AN_X); vec++; res = AN_Z; break;
-		default:	if (res == AN_Z) return(AN_X); vec++; res = AN_0; break;
+	ch = vec[i];
+        if(ch != pch) goto miscompare;
 		}
+
+return(ch);
+
+miscompare:
+if((pch == AN_X) || (pch == AN_U)) return(pch);
+if(pch == AN_Z) return(AN_X);
+for (; i < nbits; i++)
+        {
+	ch = vec[i];
+        if((ch == AN_X) || (ch == AN_U)) return(ch);
+        if(ch == AN_Z) return(AN_X);
 	}
 
-return(!an_u_encountered ? res : AN_U);
+return(AN_COUNT);
 }
 
 int vtype2(Trptr t, vptr v)
 {
-int i, nbits, res;
-int an_u_encountered = 0;
+int i, nbits;
+char pch, ch;
 char *vec=(char *)v->v;
 
 if(!t->t_filter_converted)
@@ -924,40 +940,40 @@ if(!t->t_filter_converted)
 	}
 	else
 	{
-	return ( ((vec == NULL) || (vec[0] == 0)) ? AN_Z : AN_0 );
+	return ( ((vec == NULL) || (vec[0] == 0)) ? AN_Z : AN_COUNT );
 	}
 
 nbits=t->n.vec->nbits;
-res = AN_1;
-for (i = 0; i < nbits; i++)
-	{
-	switch (*vec)
+
+pch = ch = vec[0];
+for (i = 1; i < nbits; i++)
 		{
-		case AN_X:
-		case 'x':
-		case 'X':
-				return(AN_X);
-		case AN_U:
-		case 'u':
-		case 'U':
-				an_u_encountered = 1; break;
-		case AN_Z:
-		case 'z':
-		case 'Z':
-				if (res == AN_0) return(AN_X); vec++; res = AN_Z; break;
-		default:	if (res == AN_Z) return(AN_X); vec++; res = AN_0; break;
+	ch = vec[i];
+        if(ch != pch) goto miscompare;
 		}
+
+return(ch);
+
+miscompare:
+if((pch == AN_X) || (pch == AN_U)) return(pch);
+if(pch == AN_Z) return(AN_X);
+for (; i < nbits; i++)
+        {
+	ch = vec[i];
+        if((ch == AN_X) || (ch == AN_U)) return(ch);
+        if(ch == AN_Z) return(AN_X);
 	}
 
-return(!an_u_encountered ? res : AN_U);
+return(AN_COUNT);
 }
 
+
 /*
  * convert trptr+hptr vectorstring into an ascii string
  */
 char *convert_ascii_vec_2(Trptr t, char *vec)
 {
-Ulong flags;
+TraceFlagsType flags;
 int nbits;
 char *bits;
 char *os, *pnt, *newbuff;
@@ -1409,7 +1425,16 @@ else if(flags&TR_SIGNED)
 			}
 			else
 			{
-			sprintf(os, TTFormat, val);
+                        if(!(flags&TR_TIME))
+                                {
+                                sprintf(os, TTFormat, val);
+                                }
+                                else
+                                {
+                                free_2(os);
+                                os = calloc_2(1, 128);
+                                reformat_time(os, val, GLOBALS->time_dimension);
+                                }
 			}
 		}
 		else
@@ -1507,7 +1532,16 @@ else	/* decimal when all else fails */
 			}
 			else
 			{
-			sprintf(os, UTTFormat, val);
+                        if(!(flags&TR_TIME))
+                                {
+				sprintf(os, UTTFormat, val);
+                                }
+                                else
+                                {
+                                free_2(os);
+                                os = calloc_2(1, 128);
+                                reformat_time(os, val, GLOBALS->time_dimension);
+                                }
 			}
 		}
 		else
@@ -1549,6 +1583,50 @@ if((*s == '?') && (!GLOBALS->color_active_in_filter))
 return(s);
 }
 
+static char *edofilter(Trptr t, char *s)
+{
+if(t->flags & TR_ENUM)
+	{
+	int filt = t->e_filter - 1;
+
+#ifdef _WAVE_HAVE_JUDY
+	PPvoid_t pv = JudyHSGet(GLOBALS->xl_enum_filter[filt], s, strlen(s));
+	if(pv)
+		{
+		free_2(s);
+		s = malloc_2(strlen(*pv) + 1);
+		strcpy(s, *pv);
+		}
+#else
+	GLOBALS->xl_enum_filter[filt] = xl_splay(s, GLOBALS->xl_enum_filter[filt]);
+
+	if(!strcasecmp(s, GLOBALS->xl_enum_filter[filt]->item))
+		{
+		free_2(s);
+		s = malloc_2(strlen(GLOBALS->xl_enum_filter[filt]->trans) + 1);
+		strcpy(s, GLOBALS->xl_enum_filter[filt]->trans);
+		}
+#endif
+	else
+		{
+		char *zerofind = s;
+		char *dst = s, *src;
+		while(*zerofind == '0') zerofind++;
+		if(zerofind != s)
+			{
+			src = (!*zerofind) ? (zerofind-1) : zerofind;
+			while(*src)
+				{
+				*(dst++) = *(src++);
+				}
+			*dst = 0;
+			}
+		}
+	}
+
+return(s);
+}
+
 static char *pdofilter(Trptr t, char *s)
 {
 struct pipe_ctx *p = GLOBALS->proc_filter[t->p_filter];
@@ -1624,11 +1702,16 @@ char *convert_ascii_vec(Trptr t, char *vec)
 {
 char *s = convert_ascii_vec_2(t, vec);
 
-if(!(t->f_filter|t->p_filter))
+if(!(t->f_filter|t->p_filter|t->e_filter))
 	{
 	}
 	else
 	{
+	if(t->e_filter)
+		{
+		s = edofilter(t, s);
+		}
+	else
 	if(t->f_filter)
 		{
 		s = dofilter(t, s);
@@ -1669,11 +1752,16 @@ if((!t->t_filter_converted) && (!(v->flags & HIST_STRING)))
 	        }
 	}
 
-if(!(t->f_filter|t->p_filter))
+if(!(t->f_filter|t->p_filter|t->e_filter))
 	{
 	}
 	else
 	{
+	if(t->e_filter)
+		{
+		s = edofilter(t, s);
+		}
+	else
 	if(t->f_filter)
 		{
 		s = dofilter(t, s);
